/*
 * Copyright (C) 1998-2002 <a href="mailto:jens.dietrich@unforgettable.com">Jens Dietrich</a>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.jfs2002.mandarax.crm;


import java.util.Calendar;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

/**
 * Represents customers. Note that there are some method to get the turnover.
 * These methods are used as functions to build the knowledge base.
 * (rules like: if the turnover of a customer in the last 6 month was more than
 * 500$ then ..".
 * @author <A HREF="mailto:jens.dietrich@unforgettable.com">Jens Dietrich</A>
 * @version 1.8 <18 June 2002>
 * @since 1.2
 */
public class Customer implements java.io.Serializable {

    private int    id           = 0;
    private String firstName    = "?";
    private String name         = "?";
    private Vector transactions = new Vector ();

    /**
     * Constructor.
     * @param anId int
     * @param aFirstName java.lang.String
     * @param aName java.lang.String
     */
    public Customer(int anId, String aFirstName, String aName) {
        super ();

        id        = anId;
        firstName = aFirstName;
        name      = aName;
    }

    /**
     * Add a transaction.
     * @param t Transaction
     */
    public void addTransaction(Transaction t) {
        transactions.add (t);
        t.setOwner (this);
    }

    /**
     * Get the first name.
     * @return java.lang.String
     */
    public String getFirstName() {
        return firstName;
    }

    /**
     * Get the id.
     * @return int
     */
    public int getId() {
        return id;
    }

    /**
     * Get the name.
     * @return java.lang.String
     */
    public String getName() {
        return name;
    }

    /**
     * Get the turnover during the last months.
     * @return double
     * @param numberOfMonth int
     */
    public double getTurnover(int numberOfMonth) {
        double   result   = 0;
        Calendar monthAgo = Calendar.getInstance ();

        monthAgo.add (Calendar.MONTH, -numberOfMonth);

        List transactions = DB.getTransactions (id);

        if(transactions != null) {
            for(Iterator it = transactions.iterator (); it.hasNext (); ) {
                Transaction t = (Transaction) it.next ();

                if(monthAgo.before (t.getTimeStamp ())) {
                    result = result + t.getAmount ();
                }
            }
        }

        return result;
    }

    /**
     * Get the turnover in the last numberOfMonth
     * using the kind of payment.
     * @return double
     * @param numberOfMonth int
     * @param kop KindOfPayment
     */
    public double getTurnover(int numberOfMonth, KindOfPayment kop) {
        double   result   = 0;
        Calendar monthAgo = Calendar.getInstance ();

        monthAgo.add (Calendar.MONTH, -numberOfMonth);

        List transactions = DB.getTransactions (id);

        if(transactions != null) {
            for(Iterator it = transactions.iterator (); it.hasNext (); ) {
                Transaction t = (Transaction) it.next ();

                if(monthAgo.before (t.getTimeStamp ())
                        && t.getKindOfPayment ().equals (kop)) {
                    result = result + t.getAmount ();
                }
            }
        }

        return result;
    }

    /**
     * Remove a transaction.
     * @param t Transaction
     */
    public void removeTransaction(Transaction t) {
        transactions.remove (t);
    }

    /**
     * Set the first name.
     * @param aFirstName java.lang.String
     */
    public void setFirstName(String aFirstName) {
        firstName = aFirstName;
    }

    /**
     * Set the name.
     * @param aName java.lang.String
     */
    public void setName(String aName) {
        name = aName;
    }

    /**
     * COnvert the object to a string.
     * @return java.lang.String
     */
    public String toString() {
        return firstName + " " + name;
    }
}
