/*
 * Copyright (C) 1998-2002 <a href="mailto:jens.dietrich@unforgettable.com">Jens Dietrich</a>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.jfs2002.mandarax.crm;


import java.util.Calendar;
import java.util.Iterator;
import java.util.List;
import java.util.TreeMap;
import java.util.Vector;

/**
 * Simulation of a customer database. I.e., test data are hardcoded here.
 * In a real world scenario, this class would probably be replaced by a class
 * reading (and perhaps caching) data from a RDBMS.
 * @author <A HREF="mailto:jens.dietrich@unforgettable.com">Jens Dietrich</A>
 * @version 1.8 <18 June 2002>
 * @since 1.2
 */
public class DB {

    private static TreeMap kindsOfPayment = null;
    private static TreeMap categories     = null;
    private static TreeMap customers      = null;
    private static Vector  transactions   = null;

    /**
     * Get the categories.
     * @return categories organized by name
     */
    public static TreeMap getCategories() {
        if(categories == null) {
            categories = new TreeMap ();

            categories.put (Category.PLATINUM,
                            new Category (Category.PLATINUM));
            categories.put (Category.GOLD, new Category (Category.GOLD));
            categories.put (Category.STANDARD,
                            new Category (Category.STANDARD));
        }

        return categories;
    }

    /**
     * Get a category.
     * @return a category
     * @param description a category name
     */
    public static Category getCategory(String description) {
        return(Category) getCategories ().get (description);
    }

    /**
     * Get a customer.
     * @return a customer
     * @param id a customer id
     */
    public static Customer getCustomer(int id) {
        return(Customer) getCustomers ().get (new Integer (id));
    }

    /**
     * Get the customers.
     * @return a map containing customers organized by id
     */
    public static TreeMap getCustomers() {
        if(customers == null) {
            customers = new TreeMap ();

            customers.put (new Integer (1),
                           new Customer (1, "Robert", "Musil"));
            customers.put (new Integer (2),
                           new Customer (2, "Kurt", "Vonnegut"));
            customers.put (new Integer (3),
                           new Customer (3, "Graham", "Greene"));
            customers.put (new Integer (4),
                           new Customer (4, "Thomas", "Mann"));
            customers.put (new Integer (5),
                           new Customer (5, "Umberto", "Eco"));
            customers.put (new Integer (6),
                           new Customer (6, "Ludwig", "Wittgenstein"));
        }

        return customers;
    }

    /**
     * Get a kind of payment.
     * @return the kind of payment
     * @param description the name or description
     */
    public static KindOfPayment getKindOfPayment(String description) {
        return(KindOfPayment) getKindsOfPayment ().get (description);
    }

    /**
     * Get the kinds of payment.
     * @return all kinds of payment
     */
    public static TreeMap getKindsOfPayment() {
        if(kindsOfPayment == null) {
            kindsOfPayment = new TreeMap ();

            kindsOfPayment.put (
                KindOfPayment.COMPANY_VISA,
                new KindOfPayment (KindOfPayment.COMPANY_VISA));
            kindsOfPayment.put (KindOfPayment.OTHER_VISA,
                                new KindOfPayment (KindOfPayment.OTHER_VISA));
            kindsOfPayment.put (KindOfPayment.MASTER,
                                new KindOfPayment (KindOfPayment.MASTER));
            kindsOfPayment.put (KindOfPayment.AMEX,
                                new KindOfPayment (KindOfPayment.AMEX));
            kindsOfPayment.put (KindOfPayment.DISCOVERY,
                                new KindOfPayment (KindOfPayment.DISCOVERY));
            kindsOfPayment.put (KindOfPayment.EURO,
                                new KindOfPayment (KindOfPayment.EURO));
        }

        return kindsOfPayment;
    }

    /**
     * Get the transactions.
     * @return a list of transactions
     */
    public static List getTransactions() {
        if(transactions == null) {
            transactions = new Vector ();

            Calendar date = Calendar.getInstance ();

            date.add (Calendar.DATE, -5);
            transactions.add (
                new Transaction (
                    45.50, date, getCustomer (1),
                    getKindOfPayment (KindOfPayment.COMPANY_VISA)));

            date = Calendar.getInstance ();

            date.add (Calendar.DATE, -8);
            transactions.add (
                new Transaction (
                    99.99, date, getCustomer (1),
                    getKindOfPayment (KindOfPayment.COMPANY_VISA)));

            date = Calendar.getInstance ();

            date.add (Calendar.DATE, -22);
            transactions.add (
                new Transaction (
                    35.60, date, getCustomer (1),
                    getKindOfPayment (KindOfPayment.COMPANY_VISA)));

            date = Calendar.getInstance ();

            date.add (Calendar.DATE, -89);
            transactions.add (
                new Transaction (
                    66.66, date, getCustomer (1),
                    getKindOfPayment (KindOfPayment.COMPANY_VISA)));

            date = Calendar.getInstance ();

            date.add (Calendar.DATE, -34);
            transactions.add (
                new Transaction (
                    299.95, date, getCustomer (2),
                    getKindOfPayment (KindOfPayment.AMEX)));

            date = Calendar.getInstance ();

            date.add (Calendar.DATE, -45);
            transactions.add (
                new Transaction (
                    199.50, date, getCustomer (2),
                    getKindOfPayment (KindOfPayment.AMEX)));

            date = Calendar.getInstance ();

            date.add (Calendar.DATE, -120);
            transactions.add (
                new Transaction (
                    349.50, date, getCustomer (2),
                    getKindOfPayment (KindOfPayment.AMEX)));

            date = Calendar.getInstance ();

            date.add (Calendar.DATE, -44);
            transactions.add (
                new Transaction (
                    45.50, date, getCustomer (3),
                    getKindOfPayment (KindOfPayment.EURO)));

            date = Calendar.getInstance ();

            date.add (Calendar.DATE, -185);
            transactions.add (
                new Transaction (
                    59.50, date, getCustomer (3),
                    getKindOfPayment (KindOfPayment.EURO)));

            date = Calendar.getInstance ();

            date.add (Calendar.DATE, -76);
            transactions.add (
                new Transaction (
                    32.50, date, getCustomer (4),
                    getKindOfPayment (KindOfPayment.COMPANY_VISA)));

            date = Calendar.getInstance ();

            date.add (Calendar.DATE, -34);
            transactions.add (
                new Transaction (
                    10.00, date, getCustomer (5),
                    getKindOfPayment (KindOfPayment.OTHER_VISA)));

            date = Calendar.getInstance ();

            date.add (Calendar.DATE, -192);
            transactions.add (
                new Transaction (
                    24.95, date, getCustomer (5),
                    getKindOfPayment (KindOfPayment.OTHER_VISA)));

            date = Calendar.getInstance ();

            date.add (Calendar.DATE, -88);
            transactions.add (
                new Transaction (
                    44.50, date, getCustomer (6),
                    getKindOfPayment (KindOfPayment.OTHER_VISA)));
        }

        return transactions;
    }

    /**
     * Collect and return all transactions for a certain customer.
     * @return the list of transactions of the customer witth the respective id
     * @param customerId a customer id
     */
    public static List getTransactions(int customerId) {
        List        collect = new Vector ();
        Transaction t;

        for(Iterator it = getTransactions ().iterator (); it.hasNext (); ) {
            t = (Transaction) it.next ();

            if(t.getOwner ().getId () == customerId) {
                collect.add (t);
            }
        }

        return collect;
    }
}
