/*
 * Copyright (C) 1998-2002 <a href="mailto:jens.dietrich@unforgettable.com">Jens Dietrich</a>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.jfs2002.mandarax.crm;


import java.util.Calendar;

/**
 * Represents a customer transaction.
 * @author <A HREF="mailto:jens.dietrich@unforgettable.com">Jens Dietrich</A>
 * @version 1.8 <18 June 2002>
 * @since 1.2
 */
public class Transaction implements java.io.Serializable {

    private double        amount    = 0;
    private Calendar      timeStamp = Calendar.getInstance ();
    private KindOfPayment kindOfPayment;
    private Customer      owner;

    /**
     * Constructor.
     * @param anAmount double
     * @param aDate java.util.Calendar
     * @param aCustomer Customer
     * @param aKindOfPayment KindOfPayment
     */
    public Transaction(double anAmount, Calendar aDate, Customer aCustomer,
                       KindOfPayment aKindOfPayment) {
        super ();

        setAmount (anAmount);
        setOwner (aCustomer);
        setTimeStamp (aDate);
        setKindOfPayment (aKindOfPayment);
    }

    /**
     * Get the amount of this transaction.
     * @return double
     */
    public double getAmount() {
        return amount;
    }

    /**
     * Get the kind of payment
     * @return KindOfPayment
     */
    public KindOfPayment getKindOfPayment() {
        return kindOfPayment;
    }

    /**
     * Get the customer.
     * @return Customer
     */
    public Customer getOwner() {
        return owner;
    }

    /**
     * Get the time when this transaction occured.
     * @return java.util.Calendar
     */
    public Calendar getTimeStamp() {
        return timeStamp;
    }

    /**
     * Set the amount of the transaction.
     * @param a double
     */
    public void setAmount(double a) {
        amount = a;
    }

    /**
     * Set the kind of payment.
     * @param kp KindOfPayment
     */
    public void setKindOfPayment(KindOfPayment kp) {
        kindOfPayment = kp;
    }

    /**
     * Set the customer.
     * @param c Customer
     */
    public void setOwner(Customer c) {
        owner = c;
    }

    /**
     * Set the time stamp.
     * @param aDate java.util.Calendar
     */
    public void setTimeStamp(Calendar aDate) {
        timeStamp = aDate;
    }

    /**
     * Convert the object to a string.
     * @return java.lang.String
     */
    public String toString() {
        StringBuffer buf = new StringBuffer ();

        buf.append (amount);
        buf.append ("$ - ");
        buf.append (timeStamp.get (Calendar.DAY_OF_MONTH));
        buf.append (" ");
        buf.append (timeStamp.get (Calendar.MONTH));
        buf.append (" ");
        buf.append (timeStamp.get (Calendar.YEAR));

        return new String (buf);
    }
}
